"""
This module define classes for reading and parsing data from DHT11 sensor.
"""
import itertools
import time

import RPi
import RPi.GPIO as GPIO


class SensorTimeoutError(Exception):
    """
    Timeout Exception when reading from sensor takes more than
    specified timeout.
    """


class DHT11Result:
    """DHT11 sensor result returned by DHT11.read() method"""

    ERR_NO_ERROR = 0
    ERR_MISSING_DATA = 1
    ERR_CRC = 2
    ERROR_CODES_STR = {
        0: 'No error',
        1: 'Missing data',
        2: 'CRC error'
    }

    error_code = ERR_NO_ERROR
    temperature = -1
    humidity = -1

    def __init__(self, error_code, temperature, humidity):
        self.error_code = error_code
        self.temperature = temperature
        self.humidity = humidity

    def is_valid(self):
        """ Verify that reading result contains no errors. """
        return self.error_code == DHT11Result.ERR_NO_ERROR

    def error_code_to_str(self, code):
        """ Return string representation of error code. """
        return self.ERROR_CODES_STR.get(code, 'Wrong error code')


class DHT11:
    """DHT11 sensor reader class for Raspberry"""

    __pin = 0

    def __init__(self, pin):
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BOARD)
        self.__pin = pin

    def __del__(self):
        """
        Clearing GPIO pins is required if there are multiple instances
        of reading class created.
        """
        GPIO.cleanup()

    def get_sensor_data(self, timeout_value=10):
        """
        Keep reading until valid data is returned from sensor or timeout pass.

        Args:
            timeout_value(int): timeout in seconds

        Raises:
            TimeoutError: when timeout pass before valid data is returned

        Returns:
            DHT11Result
        """
        timeout = time.time() + timeout_value
        senses = self.read()
        while not senses.is_valid():
            if time.time() > timeout:
                raise SensorTimeoutError(
                        'Timed out after {}s. Most recent error: {}'.format(
                                timeout_value,
                                senses.error_code_to_str(senses.error_code)))
            senses = self.read()
        return senses

    def temperature_to_str(self):
        """ Return string representation of temperature from sensor. """
        senses = self.get_sensor_data()
        return '{} C'.format(senses.temperature)

    def start_read(self):
        """ Init reading by sending pule through OneWire. """
        RPi.GPIO.setup(self.__pin, RPi.GPIO.OUT)
        # send initial high
        self.__send_and_sleep(RPi.GPIO.HIGH, 0.05)
        # pull down to low
        self.__send_and_sleep(RPi.GPIO.LOW, 0.02)
        # change to input using pull up
        RPi.GPIO.setup(self.__pin, RPi.GPIO.IN, RPi.GPIO.PUD_UP)

    def read(self):
        """ Read the data from OneWire and verify CRC. """
        self.start_read()

        data = self.__collect_input()
        pull_up_lengths = self.__parse_data_pull_up_lengths(data)

        if len(pull_up_lengths) != 40:
            return DHT11Result(DHT11Result.ERR_MISSING_DATA, 0, 0)

        bits = self.__calculate_bits(pull_up_lengths)
        the_bytes = self.__bits_to_bytes(bits)

        checksum = self.__calculate_checksum(the_bytes)
        if the_bytes[4] != checksum:
            return DHT11Result(DHT11Result.ERR_CRC, 0, 0)
        return DHT11Result(DHT11Result.ERR_NO_ERROR, the_bytes[2],
                           the_bytes[0])

    def __send_and_sleep(self, output, sleep):
        RPi.GPIO.output(self.__pin, output)
        time.sleep(sleep)

    def __collect_input(self):
        # collect the data while unchanged found
        unchanged_count = 0
        # this is used to determine where is the end of the data
        max_unchanged_count = 100

        last = -1
        data = []
        while unchanged_count <= max_unchanged_count:
            current = RPi.GPIO.input(self.__pin)
            data.append(current)
            if last != current:
                unchanged_count = 0
                last = current
            else:
                unchanged_count += 1
        return data

    @staticmethod
    def __parse_data_pull_up_lengths(data):
        """
        Find lenghts of highs ie __|==|__|===|__ return lengths of |==|
        (2, 3) in this case.
        """
        lengths = [len(list(x[1])) for x in itertools.groupby(data) if x[0]]
        # we need only 40 high sequences, skip first pull up/down seq
        # also skip last pull up
        return lengths[-41:-1]

    @staticmethod
    def __calculate_bits(pull_up_lengths):
        # find shortest and longest period
        shortest_pull_up = min(min(pull_up_lengths), 1000)
        longest_pull_up = max(max(pull_up_lengths), 0)

        # use the halfway to determine whether the period it is long or short
        halfway = shortest_pull_up + (longest_pull_up - shortest_pull_up) / 2
        return [length > halfway for length in pull_up_lengths]

    @staticmethod
    def __bits_to_bytes(bits):
        the_bytes = []
        for index, _ in enumerate(bits[::8]):
            section = bits[index * 8:index * 8 + 8]
            the_bytes.append(sum(value << index for index, value in
                                 enumerate(section[::-1])))
        return the_bytes

    @staticmethod
    def __calculate_checksum(the_bytes):
        return sum(the_bytes[:3])
