# pylint: disable=invalid-name
# RobotFramework handles better test library with same name as class name
"""
This module represents class that define extra methods for testing DHT11 sensor.
"""
import re
from robot.api import logger
import DHT11


class DHT11TestLibrary:
    """ Test library for Robot Framework used in testing DHT11 sensor. """
    ROBOT_LIBRARY_SCOPE = 'TEST CASE'

    def __init__(self):
        """ Init instance of class for reading from DHT11. """
        self.dht11_inst = None

    def init_dht11(self, pin):
        """ Create instance of class for reading from DHT11 and assign pin.

        Args:
            pin (int): number of pin that is connected to DHT11
        """
        self.dht11_inst = DHT11.DHT11(pin=pin)

    def dht11_shall_return_sensor_data(self, timeout=10):
        """
        Verify that DHT11 sensor return correct data before
        timeout pass.

        Args:
            timeout(int): timeout in seconds.

        Raises:
            SensorTimeoutError: when timeout pass before correct data is returned
            from sensor

        Returns:
            DHT11Result: instance of class with temperature and humidity data
        """
        senses = self.dht11_inst.get_sensor_data(timeout)
        return senses

    def dht11_shall_fail_reading_from_wrong_pin(self):
        """
        Verify that it is not possible to read from DHT11 sensor
        if the wrong pin was assigned.

        Raises:
            AssertionError: when reading from wrong ping was possible

        Returns:
            True: if dht11 module returns SensorTimeoutError
        """
        try:
            logger.info('Try to read from not connected pin', True, True)
            self.dht11_inst.get_sensor_data(5)
            raise AssertionError('DHT11 should fail reading from wrong pin')
        except DHT11.SensorTimeoutError:
            return True

    def humidity_shall_be_in_range(self, min_hum, max_hum):
        """
        Verify that humidity from sensor is in given range.

        Args:
            min_hum(int): minimum for humidity treshold (exluding)
            max_hum(int): maximum for humidity treshold (exluding)

        Raises:
            TimeoutError: when timeout pass before correct data is returned
            AssertionError: when read humidity is not between min_hum and max_hum

        Returns:
            True: when read humidity is between min_hum and max_hum
        """
        logger.info('Verify that humidity is between {} and {}'.format(min_hum, max_hum),
                    True,
                    True)
        senses = self.dht11_inst.get_sensor_data()
        logger.info('Humidity: {}'.format(senses.humidity), True, True)
        if (min_hum > senses.humidity) or (senses.humidity > max_hum):
            raise AssertionError('DHT11 returned humidity outside range')
        return True

    def temperature_to_str(self):
        """ Returns string representation of temperature from sensor."""
        return self.dht11_inst.temperature_to_str()

    def temperature_string_shall_be_in_correct_format(self):
        """
        Verify that string representation of temperature from sensor is
        in given format.

        Raises:
            AssertionError: when temperature string does not match given format

        Returns:
            True: when temperature string is in given format
        """
        temperature_str = self.dht11_inst.temperature_to_str()
        if re.match(r'^[1-9]+\d* C$', temperature_str):
            return True
        raise AssertionError('Temperature string in invalid format: {}'.format(temperature_str))
