"""
This module represents
"""
import logging
import smbus
import time

from robot.api.deco import keyword

from LcdVariables import *


class LCDTestLibrary:

    def __init__(self):
        """ Initializes LCDTestLibrary screen. """

        self.bus = smbus.SMBus(1)
        self.backlight = 0x08

        self.send_command(0x33, LCD_CMD)
        self.send_command(0x32, LCD_CMD)
        self.send_command(0x06, LCD_CMD)
        self.send_command(0x0C, LCD_CMD)
        self.send_command(0x28, LCD_CMD)

        self.send_clear_command()
        self.logger = logging.getLogger(__name__)

    def send_command(self, hex_data, mode):
        """
        Sends a command to the LCDTestLibrary screen.
        :param hex_data: value of the command to be sent
        :param mode: command mode
        :type hex_data: int
        :type mode: int 1 or 0
        """

        bits_high = mode | (hex_data & 0xF0) | self.backlight
        bits_low = mode | ((hex_data << 4) & 0xF0) | self.backlight

        # High bits
        self.bus.write_byte(I2C_ADDR, bits_high)
        self._lcd_toggle_enable(bits_high)

        # Low bits
        self.bus.write_byte(I2C_ADDR, bits_low)
        self._lcd_toggle_enable(bits_low)

    # an example of @keyword decorator used to change the keword name
    @keyword(name="Clear screen")
    def send_clear_command(self):
        """ Clears LCDTestLibrary screen."""
        self.send_command(0x01, LCD_CMD)
        time.sleep(E_DELAY)

    def enable_backlight(self):
        """ Turns on LCD backlight."""
        self.backlight = 0x08

    def disable_backlight(self):
        """ Turns off LCD backlight."""
        self.backlight = 0x00

    def display_message(self, message, line=1):
        """
        Sends a text message to be further displayed in LCD.
        :param message: message to be sent to LCD
        :param line: number of the line on which text would be displayed
        :type message: str
        :type line: int
        :return:
        """
        next_message = None

        if len(message) > 32:
            return None
        elif len(message) > 16:
            next_message = message[16:]
            message = message[:16]

        message = message.ljust(LCD_WIDTH, " ")

        if line == 1:
            line = LCD_LINE_1
        elif line == 2:
            line = LCD_LINE_2
        elif line > 2:
            return None

        self.send_command(line, LCD_CMD)

        for i in range(LCD_WIDTH):
            self.send_command(ord(message[i]), LCD_CHR)

        if line == LCD_LINE_2 and next_message:
            return None
        elif next_message:
            self.display_message(next_message, 2)

    def _lcd_toggle_enable(self, bits):
        time.sleep(E_DELAY)
        self.bus.write_byte(I2C_ADDR, (bits | ENABLE))
        time.sleep(E_PULSE)
        self.bus.write_byte(I2C_ADDR, (bits & ~ENABLE))
        time.sleep(E_DELAY)

    @staticmethod
    def user_verification():
        answer = raw_input()
        return answer
